/* Afvalalarm 2024 */
/* CC BY-NC-SA Jeroen Brinkman */
/* Versie 11: dd 30-12-2023 */

#include "RTClib.h" // zie:https://github.com/adafruit/RTClib 

RTC_DS1307 RTC;

/* pas de volgende uren aan wanneer dat gewenst is */
#define AAN1    17      // Welk uur op de dag voor de afval ophaaldag moet de LED aan
#define UIT1    23      // En op welk uur weer uit
#define AAN2     7      // Welk uur op de afval ophaaldag moet de LED aan
#define UIT2    10      // En op welk uur weer uit

#define ZOMER    1      // Bij zomertijd loopt de klok eigenlijk een uur achter, dus je moet er een uur bij optellen
#define WINTER   0      // Wintertijd is goed
#define UIT      4      // Constante met daarin de poort voor de eerste poort  

#define REBOOT   4      // Poort gebruikt om opnieuw op te starten 
#define WIT      5      // Witte LED - restafval       - 1 in de matrix LET OP: de poorten van de leds moeten opeenvolgend zijn en mogen 13 niet omvatten
#define ORANJE   6      // Gele/Oranje LED - PMD afval - 2 in de matrix
#define GROEN    7      // Groene LED - GFT afval      - 3 in de matrix
#define BLAUW    8      // Blauwe LED - oud papier     - 4 in de matrix
#define ALIVE    9      // LED die laat zien dat de klok nog loopt
#define HEART   13      // On-board heartbeat

/* LETOP: Zet TESTKLOK alleen op true om de tijd eenmalig juist te zetten. Compileer en upload daarna opnieuw met TESTKLOK is false */
#define TESTKLOK true   // Zet deze eenmalig op true als de datum en/of tijd moet worden aangepast en upload de code. Het alarm zal alleen goed werken als deze op false staat
#define CORRECTIE 0     // Zet dit op 1 wanneer de tijd gezet wordt als het zomertijd is en 0 wanneer het wintertijd is 
  
/* Dit stuk komt uit de EXCEL sheet */
#define JAAR 2024           // Het jaar van de afvalkalender
const int afval[13][31] = { // dit is de afvalkalender van 2024 met per rij de 31 dagen
{0,0,0,4,3,0,0,0,0,0,0,1,0,2,0,0,0,0,3,0,0,0,0,0,0,0,1,0,0,0,0},  // dec
{0,34,0,2,0,0,0,0,1,0,0,0,0,0,0,3,0,0,0,0,0,0,1,0,2,0,0,0,4,3,0},  // jan
{0,0,0,0,0,1,0,0,0,0,0,0,3,0,2,0,0,0,0,1,0,0,0,0,0,4,3,0,0,0,0},  // feb
{0,0,0,0,1,0,2,0,0,0,0,3,0,0,0,0,0,0,1,0,0,0,0,0,4,3,0,2,0,0,0},  // mrt
{0,1,0,0,0,0,0,0,3,0,0,0,0,0,0,1,0,2,0,0,0,4,3,0,0,0,0,0,0,1,0},  // apr
{0,0,0,0,0,0,3,2,0,0,0,0,0,1,0,0,0,0,0,0,34,0,0,0,0,0,0,1,0,2,0},  // mei
{0,0,0,3,0,0,0,0,0,0,1,0,0,0,0,0,4,3,0,2,0,0,0,0,1,0,0,0,0,0,0},  // jun
{0,3,0,0,0,0,0,0,1,0,2,0,0,0,4,3,0,0,0,0,0,0,1,0,0,0,0,0,0,3,0},  // jul
{2,0,0,0,0,1,0,0,0,0,0,4,3,0,0,0,0,0,0,1,0,2,0,0,0,0,3,0,0,0,0},  // aug
{0,0,1,0,0,0,0,0,4,3,0,2,0,0,0,0,1,0,0,0,0,0,0,3,0,0,0,0,0,0,0},  // sep
{1,0,2,0,0,0,4,3,0,0,0,0,0,0,1,0,0,0,0,0,0,3,0,2,0,0,0,0,1,0,0},  // okt
{0,0,0,4,3,0,0,0,0,0,0,1,0,2,0,0,0,0,3,0,0,0,0,0,0,1,0,0,0,0,0},  // nov
{0,4,3,0,2,0,0,0,0,1,0,0,0,0,0,0,3,0,0,0,0,0,0,1,0,0,2,0,0,4,3}};  // dec
/* Einde stuk uit de EXCEL sheet */

int TeDoen, TeDoen2, dag, maand, jaar, minuut, correctie, lastsec;
DateTime now, vuil;
unsigned long nu, check1, check2, lastmil;

void setup() {

  /* define I/O  */
  digitalWrite(REBOOT, HIGH); 
  delay(200);
  pinMode(REBOOT, OUTPUT);
  digitalWrite(REBOOT, HIGH); 

  pinMode(WIT, OUTPUT);
  pinMode(ORANJE, OUTPUT);
  pinMode(GROEN, OUTPUT);
  pinMode(BLAUW, OUTPUT);
  pinMode(ALIVE, OUTPUT);
  pinMode(HEART,  OUTPUT); // blinking led showing the programma's hartbeat
    
  /* Check de LED's */
  digitalWrite(WIT, HIGH);
  delay(500);
  digitalWrite(WIT, LOW); digitalWrite(ORANJE, HIGH);
  delay(500);
  digitalWrite(ORANJE, LOW); digitalWrite(GROEN, HIGH);
  delay(500);   
  digitalWrite(GROEN, LOW); digitalWrite(BLAUW, HIGH);
  delay(500);   
  digitalWrite(BLAUW, LOW);
  delay(500); 
     
  /* initialiseer de rtc */
  RTC.begin();
  now = RTC.now(); 

  if (TESTKLOK) {
    /* initialiseer de seriele poort */
    Serial.begin(9600);
    Wire.begin();
    Serial.println(""); Serial.println("--------- Begin ---------");
    Serial.print("Oude datum: "); Serial.print(now.day(), DEC); Serial.print("/"); Serial.print(now.month(), DEC); Serial.print("/"); Serial.println(now.year(), DEC);
    Serial.print("Oude tijd: "); Serial.print(now.hour(), DEC); Serial.print(":"); if (now.minute() < 10) Serial.print("0"); Serial.println(now.minute(), DEC);

    if (!RTC.isrunning()) {
      Serial.println("RTC loopt niet!");
      RTC.adjust(DateTime(__DATE__, __TIME__)); //Zet de datum en tijd gelijk aan de compiledatum en -tijd
      now = RTC.now(); 
      Serial.print("Nieuwe datum: "); Serial.print(now.day(), DEC); Serial.print("/"); Serial.print(now.month(), DEC); Serial.print("/"); Serial.println(now.year(), DEC);
      Serial.print("Nieuwe tijd: "); Serial.print(now.hour(), DEC); Serial.print(":"); if (now.minute() < 10) Serial.print("0"); Serial.println(now.minute(), DEC);
    }
  
    Serial.println("RTC wordt op de juiste tijd gezet");
    RTC.adjust(DateTime(__DATE__, __TIME__)); //Zet de datum en tijd gelijk aan de compiledatum en -tijd
    now = RTC.now(); 
    Serial.print("Nieuwe datum: "); Serial.print(now.day(), DEC); Serial.print("/"); Serial.print(now.month(), DEC); Serial.print("/"); Serial.println(now.year(), DEC);
    Serial.print("Nieuwe tijd: "); Serial.print(now.hour(), DEC); Serial.print(":"); if (now.minute() < 10) Serial.print("0"); Serial.println(now.minute(), DEC);
  
    /* print de afhaaldata */
    Serial.println("");
    Serial.println("Gevonden afhaaldata");
    TeDoen = 0; maand = 0; dag = 1;
    while (maand != 13){
      TeDoen = afval[(maand)][(dag - 1)];
      if (TeDoen != 0) {
	    if (dag < 10) Serial.print(" "); Serial.print(dag); Serial.print("/"); 
        if ((maand < 10) && (maand > 0)) Serial.print("0"); (maand == 0) ? Serial.print("12") : Serial.print(maand); Serial.print("/"); 
        (maand == 0) ? Serial.print(JAAR-1) : Serial.print(JAAR); Serial.print(" -> ");
        switch (TeDoen) {
        case 1:
          Serial.print("Grijze bak - restafval");
          break;
        case 2:
          Serial.print("Oranje bak - PMT afval");
          break;
        case 3:
          Serial.print("Groene bak - GFT afval");
          break;
        case 4:
          Serial.print("Blauwe bak - Papier en karton");
          break;
        }
        switch (TeDoen/10) {
        case 1:
          Serial.print("Grijze bak - restafval");
          break;
        case 2:
          Serial.print("Oranje bak - PMT afval");
          break;
        case 3:
          Serial.print("Groene bak - GFT afval");
          break;
        case 4:
          Serial.print("Blauwe bak - Papier en karton");
          break;
        }
        if (TeDoen > 10) {
          TeDoen = TeDoen/10; // splits de containers
          switch (TeDoen) {
          case 1:
            Serial.print(" & Grijze bak - restafval");
            break;
          case 2:
            Serial.print(" & Oranje bak - PMT afval");
            break;
          case 3:
            Serial.print(" & Groene bak - GFT afval");
            break;
          case 4:
            Serial.print(" & Blauwe bak - Papier en karton");
            break;
          }
        }
        Serial.println("");
      }
      dag = dag + 1; 
      if (dag == 32) {   
        maand = maand + 1; 
        dag = 1;
      } 
    }

  }
  digitalWrite(WIT, HIGH); digitalWrite(ORANJE, HIGH); digitalWrite(GROEN, HIGH); digitalWrite(BLAUW, HIGH); digitalWrite(ALIVE, HIGH);
  delay(5000); // Wait 10 seconds before activating the watchdog to be sure that new software can be loaded
  digitalWrite(WIT, LOW); digitalWrite(ORANJE, LOW); digitalWrite(GROEN, LOW); digitalWrite(BLAUW, LOW); digitalWrite(ALIVE, LOW);
}

void loop() {
  digitalWrite(HEART, (millis() / 1000) % 2); //1s heartbeat voor de Arduino LED
  now = RTC.now(); 
  
  /* loopt de klok nog */ 
  if (!RTC.isrunning()) digitalWrite(REBOOT, LOW); // RTC is not running, wait three seconds and therewith initiate the watchdog reset.
  if (lastsec != now.second()){ // the clock has advanced a second
    lastmil = millis() + 2000; // two seconds from now must never occur
    lastsec = now.second();
  } 
  if (millis() > lastmil) digitalWrite(REBOOT, LOW); // if the clock has not advanced within 2 seconds, wait three seconds and therewith initiate the watchdog reset.
  if ((now.hour() == 2) && (now.minute() == 2) && (now.second() == 2)) digitalWrite(REBOOT, LOW); // reset the clock every night at 02:02:02

  /* bereken zomertijd correctie */ 
  vuil = DateTime(now.year(), 4, 1, 1, 1, 1); // 1 april yyyy 1:01:01
  check1 = 33102L - ((vuil.dayOfTheWeek() - 1) * 100); // last sunday of march march in format mmddhh
  vuil = DateTime(now.year(), 11, 1, 1, 1, 1); // 1 november yyyy 1:01:01
  check2 = 103103L - ((vuil.dayOfTheWeek() - 1) * 100); // last sunday of october in format mmddhh
  nu = (now.month()*100); nu = nu * 100; nu = nu + (now.day()*100) + now.hour(); // other construction gives wrong value during run-time
  correctie = ZOMER; if ((nu < check1) || (nu >= check2)) correctie = WINTER; 
  nu = now.hour() + correctie - CORRECTIE; if (nu == 24) nu = 0; // corrigeer het uur voor de zomertijd
  
  TeDoen = 0; TeDoen2 = 0; 
  maand = now.month(); if (now.year() == (JAAR - 1)) maand = 0; // zet de maand op 0 als het vorig jaar is
 
  /* Moet de LED op de vuilophaaldag aan */
  if (now.year() == JAAR) TeDoen = afval[maand][(now.day() - 1)]; // Check of er afval aan de weg moet staan, het array begint bij 0
  if ((nu < AAN2) || (nu >= UIT2)) TeDoen = 0; // Als de LED niet moet branden, zet hem dan weer op nul
  
  /* Moet de LED op de dag voor de vuilophaaldag aan */
  vuil = (now.unixtime() + 86400L); // een dag later
  maand = vuil.month(); if ((maand == 12) && (vuil.year() == (JAAR - 1))) maand = 0; // december JAAR-1 zit in rij 0
  if (TeDoen == 0) { // Als de led niet brandt voor vandaag, kijk dan of hij voor morgen moet branden
    TeDoen = afval[maand][(vuil.day() - 1)]; // Check of er afval aan de weg moet staan, het array begint bij 0
    if ((nu < AAN1) || (nu >= UIT1)) TeDoen = 0; // Als de LED niet moet branden, zet hem dan weer op nul
  }

  /* Bepaal of er een dubbele ophaaldag is */
  if (TeDoen > 10) {
    TeDoen2 = TeDoen/10; // splits de containers
    TeDoen = TeDoen - (TeDoen2*10);
  }

  /* zet de led aan of uit */
  if (TeDoen == 0) { 
    digitalWrite(WIT, LOW); // zet de leds uit
    digitalWrite(ORANJE, LOW);
    digitalWrite(GROEN, LOW);
    digitalWrite(BLAUW, LOW);
  } else { 
    digitalWrite((UIT + TeDoen), HIGH); // zet de betreffende led aan
    if (TeDoen2 > 0) digitalWrite((UIT + TeDoen2), HIGH); // zet de betreffende led aan
  } 
       
  if (!RTC.isrunning()) { //alle LED's gaan knipperen wanneer de klok stuk is
    digitalWrite(WIT, (millis() / 1000) % 2); 
    digitalWrite(ORANJE, (millis() / 1000) % 2); 
    digitalWrite(GROEN, (millis() / 1000) % 2); 
    digitalWrite(BLAUW, (millis() / 1000) % 2);  
  }
  
  /* laat zien dat de klok loopt */
  if (minuut != now.minute()) {
    if (TESTKLOK) {
      Serial.print("Actuele tijd: "); Serial.print(nu, DEC); Serial.print(":"); if (now.minute() < 10) Serial.print("0"); Serial.println(now.minute(), DEC);
    }
    minuut = now.minute();
    digitalWrite(ALIVE, HIGH); delay(100); digitalWrite(ALIVE, LOW);
  }
  
}
